<?php

/**
 * The admin-specific functionality of the plugin.
 *
 * @link       https://https://themeatelier.net
 * @since      1.0.0
 *
 * @package    Catshowcase_Pro
 * @subpackage Catshowcase_Pro/admin
 */

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    Catshowcase_Pro
 * @subpackage Catshowcase_Pro/admin
 * @author     ThemeAtelier <themeatelierbd@gmail.com>
 */
class Catshowcase_Pro_Admin
{

	protected $suffix;
	private $plugin_name;
	private $version;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string    $plugin_name       The name of this plugin.
	 * @param      string    $version    The version of this plugin.
	 */
	public function __construct($plugin_name, $version)
	{

		$this->suffix      = defined('SCRIPT_DEBUG') && SCRIPT_DEBUG || defined('WP_DEBUG') && WP_DEBUG ? '' : '.min';
		$this->plugin_name = $plugin_name;
		$this->version = $version;

		// Autoload system.
		spl_autoload_register(array($this, 'autoload'));

		CATSP_Metaboxes::layout_metabox('ta_catsp_layouts');
		CATSP_Metaboxes::option_metabox('ta_catsp_view_options');
		CATSP_Metaboxes::shortcode_metabox('ta_catsp_display_shortcode');
		CATSP_Settings::settings('ta_catshowcase_settings');

		CATSP_Metaboxes::cat_metabox('ta_cat_metabox');

		$active_plugins = get_option('active_plugins');
		foreach ($active_plugins as $active_plugin) {
			$_temp = strpos($active_plugin, 'catshowcase-pro.php');
			if (false != $_temp) {
				add_filter('plugin_action_links_' . $active_plugin, array($this, 'add_plugin_action_links'));
			}
		}
	}

	public function add_plugin_action_links($links)
	{
		$new_links = array(
			sprintf('<a href="%s">%s</a>', admin_url('post-new.php?post_type=catshowcase'), esc_html__('Add New', 'catshowcase-pro')),
			sprintf('<a href="%s">%s</a>', admin_url('edit.php?post_type=catshowcase'), esc_html__('Settings', 'catshowcase-pro')),
		);
		return array_merge($new_links, $links);
	}

	/**
	 * Autoload class files on demand
	 *
	 * @param string $class requested class name.
	 * @since 2.2.0
	 */
	private function autoload($class)
	{
		$name = explode('_', $class);
		if (isset($name[1])) {
			$class_name       = strtolower($name[1]);
			$catsp_config_paths = array('views/', 'views/configs/settings/', 'views/configs/generator/');
			foreach ($catsp_config_paths as $ta_catsp_path) {
				$filename = plugin_dir_path(__FILE__) . '/' . $ta_catsp_path . 'class-catsp-' . $class_name . '.php';
				if (file_exists($filename)) {
					require_once $filename;
				}
			}
		}
	}

	/**
	 * Register the stylesheets for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_styles()
	{

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in Catshowcase_Pro_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The Catshowcase_Pro_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */

		$current_screen        = get_current_screen();
		$the_current_post_type = $current_screen->post_type;
		if ('catshowcase' == $the_current_post_type) {
			wp_enqueue_style('catshowcase-admin', CATSP_URL . 'admin/assets/css/catshowcase-pro-admin' . $this->suffix . '.css', array(), $this->version, 'all');
		}
	}

	/**
	 * Register the JavaScript for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_scripts()
	{

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in Catshowcase_Pro_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The Catshowcase_Pro_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */

		$current_screen        = get_current_screen();
		$the_current_post_type = $current_screen->post_type;
		if ('catshowcase' == $the_current_post_type) {
			wp_enqueue_script('catshowcase-admin', CATSP_URL . 'admin/assets/js/catshowcase-pro-admin' . $this->suffix . '.js', array('jquery'), $this->version, false);
		}
	}

	/**
	 * Add catshowcase admin columns.
	 *
	 * @since 2.0.0
	 * @return statement
	 */
	public function filter_catshowcase_admin_column()
	{

		$admin_columns['cb']         = '<input type="checkbox" />';
		$admin_columns['title']      = esc_html__('Title', 'catshowcase-pro');
		$admin_columns['shortcode']  = esc_html__('Shortcode', 'catshowcase-pro');
		$admin_columns['catsp_layout'] = esc_html__('Layout', 'catshowcase-pro');
		$admin_columns['date']       = esc_html__('Date', 'catshowcase-pro');

		return $admin_columns;
	}

	/**
	 * Display admin columns for the catshowcases.
	 *
	 * @param mix    $column The columns.
	 * @param string $product_id The post ID.
	 * @return void
	 */
	public function display_catshowcase_admin_fields($column, $product_id)
	{

		$catsp_layouts     = get_post_meta($product_id, 'ta_catsp_layouts', true);
		$catshowcases_types = isset($catsp_layouts['catsp_layout_preset']) ? $catsp_layouts['catsp_layout_preset'] : '';
		switch ($column) {
			case 'shortcode':
				$allowed_tags = array(
					'input' => array(
						'class' => array(),
						'style' => array(),
						'type' => array(),
						'onClick' => array(),
						'readonly' => array(),
						'value' => array(),
					),
					'div' => array(
						'class' => array(),
					),
					'i' => array(
						'class' => array(),
					),
				);
				$column_field = '<input  class="ta_catsp_input" style="width: 230px;padding: 4px 8px;cursor: pointer;" type="text" onClick="this.select();" readonly="readonly" value="[catshowcase id=&quot;' . esc_attr($product_id) . '&quot;]"/> <div class="catsp-after-copy-text"><i class="far fa-check-circle"></i> ' . esc_html('Shortcode Copied to Clipboard!', 'catshowcase-pro') . ' </div>';
				echo wp_kses($column_field, $allowed_tags);
				break;
			case 'catsp_layout':
				$layout = ucwords(str_replace('_layout', ' ', $catshowcases_types));
				echo esc_html($layout);
				break;
		} // end switch.
	}

	/**
	 * If it is the plugins page.
	 *
	 * @since 2.2.0
	 * @access private
	 */
	private function is_plugins_screen()
	{
		return in_array(get_current_screen()->id, ['plugins', 'plugins-network']);
	}
}

/**
 * Catshowcase dashboard capability.
 *
 * @return string
 */
function catsp_dashboard_capability()
{
	return apply_filters('catsp_dashboard_capability', 'manage_options');
}
