<?php

/**
 * The file that defines the core plugin class
 *
 * A class definition that includes attributes and functions used across both the
 * public-facing side of the site and the admin area.
 *
 * @link       https://https://themeatelier.net
 * @since      1.0.0
 *
 * @package    Catshowcase_Pro
 * @subpackage Catshowcase_Pro/includes
 */

/**
 * The core plugin class.
 *
 * This is used to define internationalization, admin-specific hooks, and
 * public-facing site hooks.
 *
 * Also maintains the unique identifier of this plugin as well as the current
 * version of the plugin.
 *
 * @since      1.0.0
 * @package    Catshowcase_Pro
 * @subpackage Catshowcase_Pro/includes
 * @author     ThemeAtelier <themeatelierbd@gmail.com>
 */
class Catshowcase_Pro {

	/**
	 * The loader that's responsible for maintaining and registering all hooks that power
	 * the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      Catshowcase_Pro_Loader    $loader    Maintains and registers all hooks for the plugin.
	 */
	protected $loader;

	/**
	 * The unique identifier of this plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $plugin_name    The string used to uniquely identify this plugin.
	 */
	protected $plugin_name;

	/**
	 * The current version of the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $version    The current version of the plugin.
	 */
	protected $version;

	/**
	 * Define the core functionality of the plugin.
	 *
	 * Set the plugin name and the plugin version that can be used throughout the plugin.
	 * Load the dependencies, define the locale, and set the hooks for the admin area and
	 * the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function __construct() {
		$this->define_constants();
		$this->load_dependencies();
		$this->define_common_hooks();
		$this->define_admin_hooks();
		$this->define_public_hooks();
		$this->set_locale();
	}

	/**
	 * Define constant if not already set
	 *
	 * @since 2.2.0
	 *
	 * @param string      $name Define constant.
	 * @param string|bool $value Define constant.
	 */
	public function define( $name, $value ) {
		if ( ! defined( $name ) ) {
			define( $name, $value );
		}
	}

	/**
	 * Define constants
	 *
	 * @since 2.2.0
	 */
	public function define_constants() {
		$this->define( 'CATSP_VERSION', $this->version );
		$this->define( 'CATSP_PLUGIN_NAME', $this->plugin_name );
		$this->define( 'CATSP_PATH', plugin_dir_path( __DIR__ ) );
		$this->define( 'CATSP_TEMPLATE_PATH', plugin_dir_path( __DIR__ ) . 'public/templates/' );
		$this->define( 'CATSP_URL', plugin_dir_url( __DIR__ ) );
		$this->define( 'CATSHOWCASE_PRO_BASENAME', CATSHOWCASE_PRO_BASENAME );
	}

	/**
	 * Load the required dependencies for this plugin.
	 *
	 * Include the following files that make up the plugin:
	 *
	 * - Catshowcase_Pro_Loader. Orchestrates the hooks of the plugin.
	 * - Catshowcase_Pro_i18n. Defines internationalization functionality.
	 * - Catshowcase_Pro_Admin. Defines all hooks for the admin area.
	 * - Catshowcase_Pro_Public. Defines all hooks for the public side of the site.
	 *
	 * Create an instance of the loader which will be used to register the hooks
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function load_dependencies() {

		/**
		 * The class responsible for orchestrating the actions and filters of the
		 * core plugin.
		 */
		require_once CATSP_PATH . 'includes/class-catshowcase-pro-loader.php';

		/**
		 * The class responsible for defining internationalization functionality
		 * of the plugin.
		 */
		require_once CATSP_PATH . 'includes/class-catshowcase-pro-post-types.php';

		/**
		 * The class responsible for defining internationalization functionality
		 * of the plugin.
		 */
		require_once CATSP_PATH . 'includes/class-catshowcase-pro-i18n.php';

		require_once CATSP_PATH . 'admin/views/ta-framework/classes/setup.class.php';

		/**
		 * The class responsible for defining all actions that occur in the admin area.
		 */
		require_once CATSP_PATH . 'admin/class-catshowcase-pro-admin.php';

		/**
		 * The class responsible for defining metabox config that occur in the admin area.
		 */
		require_once CATSP_PATH . 'admin/helpers/class-catshowcase-pro-image-resizer.php';

		/**
		 * The class responsible for defining all actions that occur in the public-facing
		 * side of the site.
		 */
		require_once CATSP_PATH . 'public/class-catshowcase-pro-public.php';

		$this->loader = new Catshowcase_Pro_Loader();

	}

	/**
	 * Define the locale for this plugin for internationalization.
	 *
	 * Uses the Catshowcase_Pro_i18n class in order to set the domain and to register the hook
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function set_locale() {

		$plugin_i18n = new Catshowcase_Pro_i18n();

		$this->loader->add_action( 'plugins_loaded', $plugin_i18n, 'load_plugin_textdomain' );

	}

	/**
	 * Register common hooks.
	 *
	 * @since 1.0.0
	 * @access private
	 */
	private function define_common_hooks() {
		$common_hooks = new Catshowcase_pro_Post_Type( CATSP_PLUGIN_NAME, CATSP_VERSION );
		$this->loader->add_action( 'init', $common_hooks, 'register_catshowcase_post_type', 10 );
	}

	/**
	 * Register all of the hooks related to the admin area functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_admin_hooks() {

		$plugin_admin = new Catshowcase_Pro_Admin( $this->get_plugin_name(), $this->get_version() );

		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_styles' );
		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_scripts' );

		$this->loader->add_filter( 'manage_catshowcase_posts_columns', $plugin_admin, 'filter_catshowcase_admin_column' );
		$this->loader->add_action( 'manage_catshowcase_posts_custom_column', $plugin_admin, 'display_catshowcase_admin_fields', 10, 2 );
	}

	/**
	 * Register all of the hooks related to the public-facing functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_public_hooks() {

		$plugin_public = new Catshowcase_Pro_Public( $this->get_plugin_name(), $this->get_version() );

		$this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_styles' );
		$this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_scripts' );

	}

	/**
	 * Run the loader to execute all of the hooks with WordPress.
	 *
	 * @since    1.0.0
	 */
	public function run() {
		$this->loader->run();
	}

	/**
	 * The name of the plugin used to uniquely identify it within the context of
	 * WordPress and to define internationalization functionality.
	 *
	 * @since     1.0.0
	 * @return    string    The name of the plugin.
	 */
	public function get_plugin_name() {
		return $this->plugin_name;
	}

	/**
	 * The reference to the class that orchestrates the hooks with the plugin.
	 *
	 * @since     1.0.0
	 * @return    Catshowcase_Pro_Loader    Orchestrates the hooks of the plugin.
	 */
	public function get_loader() {
		return $this->loader;
	}

	/**
	 * Retrieve the version number of the plugin.
	 *
	 * @since     1.0.0
	 * @return    string    The version number of the plugin.
	 */
	public function get_version() {
		return $this->version;
	}

}
