<?php

/**
 * The file functions.
 *
 * @package Catshowcase_Pro
 * @subpackage Catshowcase_Pro/public/helper
 *
 * @since 2.0.0
 */

/**
 * Post views helper method.
 *
 * @since 2.0.0
 */
class CATSP_Functions
{

	/**
	 * WP Version compare
	 *
	 * @param  mixed $version_to_compare version compare.
	 * @param  mixed $operator operator.
	 * @return statement
	 */
	public static function wp_version_compare($version_to_compare, $operator = '>=')
	{
		if (empty($version_to_compare)) {
			return true;
		}
		global $wp_version;

		// Check if using WordPress version 3.7 or higher.
		return version_compare($wp_version, $version_to_compare, $operator);
	}

	/**
	 * Post title character limit.
	 *
	 * @param string  $catsp_title The post title.
	 * @param integer $limit_length The length for the title.
	 * @param string  $catsp_after_string The string after title.
	 * @return statement
	 */
	public static function limit_post_title($catsp_title, $limit_length, $catsp_after_string = '...')
	{
		return mb_strimwidth($catsp_title, 0, $limit_length, apply_filters('catsp_product_category_name_ellipsis', $catsp_after_string));
	}

	/**
	 * Tag name to full tag conversion.
	 *
	 * @param array $meta_tag Tag option.
	 * @return string
	 */
	public static function short_tag_to_html($meta_tag)
	{
		$exclude_tag_string = '';
		foreach ($meta_tag as $key => $value) {
			$exclude_tag_string .= '<' . $value . '>,';
		}
		return $exclude_tag_string;
	}

	/**
	 * Limit the the text.
	 *
	 * @param mixed  $text The text you want to limit.
	 * @param int    $limit The number of words to display.
	 * @param string $ellipsis The ellipsis at the end of the text.
	 * @return statement
	 */
	public static function catsp_limit_text($text, $limit, $ellipsis = '...')
	{
		$word_arr = explode(' ', $text);
		if (count($word_arr) > $limit) {
			$words = implode(' ', array_slice($word_arr, 0, $limit)) . $ellipsis;
			return $words;
		} else {
			return $text;
		}
	}

	/**
	 * Characters Limit of the text.
	 *
	 * @param mixed  $text The text you want to limit.
	 * @param int    $limit The number of words to display.
	 * @param string $ellipsis The ellipsis at the end of the text.
	 * @return statement
	 */
	public static function limit_content_chr($text, $limit, $ellipsis = '...')
	{
		$length = $limit;
		if (strlen($text) < $length + 10) {
			return $text; // don't cut if too short.
		}
		$visible = substr($text, 0, $length);
		return $visible . $ellipsis;
	}

	/**
	 * Allowed tags function of the Plugin.
	 *
	 * @since 2.0.0
	 * @return array allowed tags
	 */
	public static function allowed_tags()
	{
		$allowed_tags           = wp_kses_allowed_html('post');
		$allowed_tags['iframe'] = array(
			'src'             => array(),
			'height'          => array(),
			'width'           => array(),
			'frameborder'     => array(),
			'allowfullscreen' => array(),
			'title'           => array(),
			'alt'             => array(),
		);

		$allowed_tags['style'] = array();

		return $allowed_tags;
	}

	/**
	 * Content function.
	 *
	 * @param array  $view_options Button options array.
	 * @param string $type Content type.
	 * @param  mixed  $product post.
	 * @return content
	 */
	public static function catsp_content($view_options, $product)
	{
		$catsp_content_length_type      = isset($view_options['catsp_content_length_type']) ? $view_options['catsp_content_length_type'] : 'words';
		$product_content_length       = isset($view_options['catsp_content_limit']) ? $view_options['catsp_content_limit'] : '';
		$catsp_content_characters_limit = isset($view_options['catsp_content_characters_limit']) ? $view_options['catsp_content_characters_limit'] : '';
		$product_content_ellipsis     = isset($view_options['post_content_ellipsis']) ? $view_options['post_content_ellipsis'] : '';

		$is_page_content = false;
		$is_page_content = apply_filters('catsp_strip_shortcode_in_page_content', $is_page_content);
		global $wp_embed;


		if ($is_page_content) {
			$product_content = apply_filters('ta_catsp_the_content', strip_shortcodes($product->description));
		} else {
			$product_content = apply_filters('ta_catsp_the_content', $product->description);
		}
		if ('characters' === $catsp_content_length_type) {
			self::limit_content_chr($product_content, $catsp_content_characters_limit, $product_content_ellipsis);
		} else {
			$_trimmed_content = self::catsp_limit_text($product_content, $product_content_length, $product_content_ellipsis);
		}
		
		$catsp_product_content = $_trimmed_content;
		
		$catsp_product_content = force_balance_tags($catsp_product_content);

		$catsp_product_content = do_shortcode($wp_embed->autoembed($catsp_product_content));
		return $catsp_product_content;
	}

	/**
	 * Get the ID by the Image URL.
	 *
	 * @param mixed $url The image url.
	 * @param int   $product_id The image url.
	 * @return statement
	 */
	public static function catsp_image_id_by_url($url, $product_id = null)
	{
		global $wpdb;

		// If the URL is auto-generated thumbnail, remove the sizes and get the URL of the original image.
		$url       = preg_replace('/-\d+x\d+(?=\.(png|jp(e|g|eg)|gif|bmp|ico|webp|svg)$)/i', '', $url);
		$cache_key = 'catsp_content_img_url_to_id' . $product_id;
		$img_cache = wp_cache_get($cache_key);
		if (false === $img_cache) {
			$image = $wpdb->get_col($wpdb->prepare("SELECT ID FROM $wpdb->posts WHERE guid=%s;", $url));
			wp_cache_set($cache_key, $img_cache);
		}
		if (!empty($image)) {
			return $image[0];
		}
		return false;
	}
	/**
	 * Thumb alter text
	 *
	 * @param integer $slide_id The slide/post ID.
	 *
	 * @return string
	 */
	public static function catsp_thumb_alter_text($slide_id)
	{
		$image_id = get_post_thumbnail_id($slide_id);
		$alt_text = get_post_meta($image_id, '_wp_attachment_image_alt', true);
		return $alt_text;
	}

	/**
	 * Modify query params
	 *
	 * @param  array  $query_args query_args.
	 * @param  string $keyword keyword.
	 * @param  array  $custom_fields_array custom fields array.
	 * @param  string $orderby orderby.
	 * @param  string $order order.
	 * @param  array  $selected_term_list term_list.
	 * @param  int    $product_offset offset.
	 * @param  string $relation relation operator.
	 * @param  array  $product_in post in.
	 * @param  string $lang current lang.
	 * @return array
	 */
	public static function modify_query_params($query_args, $keyword, $custom_fields_array, $orderby, $order, $selected_term_list, $product_offset, $relation, $product_in = array(), $lang = '')
	{
		if (!empty($keyword)) {
			$query_args['s'] = $keyword;
			$product_offset  = 0;
		}
		if (!empty($lang)) {
			$query_args['lang'] = $lang;
		}
		$query_args['include'] = $product_in;
		if (!empty($custom_fields_array)) {
			$catsp_meta_query = array();
			foreach ($custom_fields_array as $_custom_field) {
				$c_field_key   = $_custom_field['custom_field_key'];
				$c_field_value = $_custom_field['custom_field_value'];
				if (strpos($c_field_value, ' ') !== false) {
					$c_field_value    = explode(' ', $c_field_value);
					$catsp_meta_query[] = array(
						'key'     => $c_field_key,
						'value'   => $c_field_value,
						'compare' => 'BETWEEN',
						'type'    => 'numeric',
					);
				} else {
					if (strpos($c_field_value, ',') !== false) {
						$c_field_value = explode(',', $c_field_value);
					}
					$catsp_meta_query[] = array(
						'key'     => $c_field_key,
						'compare' => 'IN',
						'value'   => $c_field_value,
					);
				}
			}
			if (count($catsp_meta_query) > 1) {
				$catsp_meta_query['relation'] = 'AND';
			}
			$query_args['meta_query'] = $catsp_meta_query;
		}

		if (!empty($orderby)) {
			$query_args['orderby'] = ('rand' === $orderby) ? 'rand(' . get_transient('catsp_rand') . ')' : $orderby;
		}
		if (!empty($order)) {
			$query_args['order'] = $order;
		}
		if (!empty($selected_term_list)) {
			if (count($selected_term_list) > 1) {
				$tax_settings['relation'] = $relation;
			}
			if (is_array($selected_term_list)) {
				foreach ($selected_term_list as $key => $tax_type) {
					$taxonomy = $tax_type['taxonomy'];
					$terms    = $tax_type['term_id'];
					if (strpos($terms, ',') !== false) {
						$terms = explode(',', $terms);
					}
					if ($taxonomy && $terms) {
						$tax_settings[] = array(
							'taxonomy'         => $taxonomy,
							'field'            => 'term_id',
							'terms'            => $terms,
							'operator'         => 'IN',
							'include_children' => false,
						);
					}
				}
				$query_args['tax_query'] = $tax_settings;
			}
			$product_offset = 0;
		}
		$query_args['offset'] = $product_offset;
		return $query_args;
	}

	/**
	 * Thumb Sized function
	 *
	 * @param array   $product_thumb_setting Thumbnails options array.
	 * @param integer $slide_id The slide/post ID.
	 * @param  bool    $is_attachment type attachment.
	 *
	 * @return string
	 */
	public static function catsp_sized_thumb($product_thumb_setting, $slide_id, $is_attachment = false)
	{
		$thumb_id                     = '';
		$image                        = '';
		if (!empty(get_term_meta($slide_id, 'thumbnail_id', true))) {
			$thumb_id = get_term_meta($slide_id, 'thumbnail_id', true);
		}

		
			

		

		$show_2x_image                = isset($product_thumb_setting['load_2x_image']) ? $product_thumb_setting['load_2x_image'] : false;
		$image_resize_2x_url          = '';

		$placeholder_img = CATSP_URL . 'public/assets/img/placeholder.png';
		$placeholder_img = apply_filters('catsp_no_thumb_placeholder', $placeholder_img);

		if (empty($thumb_id) && !empty($placeholder_img)) {
			$thumb_id = attachment_url_to_postid($placeholder_img);
		}
		if ($is_attachment) {
			$thumb_id = $slide_id;
		}
		if (!empty($thumb_id)) {
			$image_sizes          = isset($product_thumb_setting['catsp_thumb_sizes']) ? $product_thumb_setting['catsp_thumb_sizes'] : 'large';
			$product_image_width  = isset($product_thumb_setting['catsp_image_crop_size']['top']) ? $product_thumb_setting['catsp_image_crop_size']['top'] : '';
			$product_image_height = isset($product_thumb_setting['catsp_image_crop_size']['right']) ? $product_thumb_setting['catsp_image_crop_size']['right'] : '';
			$product_image_crop   = isset($product_thumb_setting['catsp_image_crop_size']['style']) ? $product_thumb_setting['catsp_image_crop_size']['style'] : '';
			$thumb_full_src       = wp_get_attachment_image_src($thumb_id, 'full');
			$thumb_full_src       = is_array($thumb_full_src) ? $thumb_full_src : array('', '', '');
			$image_src            = wp_get_attachment_image_src($thumb_id, $image_sizes);
			$image_src            = is_array($image_src) ? $image_src : array('', '', '');

			if (('custom' === $image_sizes) && (!empty($product_image_width) && $thumb_full_src[1] >= $product_image_width) && (!empty($product_image_height) && $thumb_full_src[2] >= $product_image_height)) {
				$hard_crop = 'Hard-crop' === $product_image_crop ? true : false;

				$image = ta_catsp_resize($thumb_full_src[0], $product_image_width, $product_image_height, $hard_crop);
				if ($show_2x_image && ($thumb_full_src[1] >= ($product_image_width * 2)) && $thumb_full_src[2] >= ($product_image_height * 2)) {
					$image_resize_2x_url = ta_catsp_resize($thumb_full_src[0], $product_image_width * 2, $product_image_height * 2, $hard_crop);
				} elseif ($show_2x_image && (($product_image_width * 2) === $thumb_full_src[1]) && ($product_image_height * 2) === $thumb_full_src[2]) {
					$image_resize_2x_url = $thumb_full_src[0];
				}
				$image_width  = $product_image_width;
				$image_height = $product_image_height;
			} else {
				$image        = !empty($image_src[0]) ? $image_src[0] : $placeholder_img;
				$image_width  = !empty($image_src[1]) ? $image_src[1] : 600;
				$image_height = !empty($image_src[2]) ? $image_src[2] : 450;
			}
		} else {
			$image        = $placeholder_img;
			$image_width  = 600;
			$image_height = 450;
		}
		$catsp_image_attr = array(
			'src'        => $image,
			'2x_src'     => $image_resize_2x_url,
			'width'      => $image_width,
			'height'     => $image_height,
			'aria_label' => 'feature_image',
		);

		return $catsp_image_attr;
	}


	/**
	 * Thumb Sized function
	 *
	 * @param  array $view_options options array.
	 * @param  int   $slide_id post id.
	 * @param  bool  $is_attachment type attachment.
	 * @return array
	 */
	public static function catsp_sized_catspup_thumb($view_options, $slide_id, $is_attachment = false)
	{
		$thumb_id     = '';
		$image        = '';
		$image_height = '';
		$image_width  = '';
		if (has_post_thumbnail($slide_id)) {
			$thumb_id = get_post_thumbnail_id($slide_id);
		}
		if ($is_attachment) {
			$thumb_id = $slide_id;
		}
		if (!empty($thumb_id)) {
			$image_sizes          = isset($view_options['catsp_catspup_img_sizes']) ? $view_options['catsp_catspup_img_sizes'] : 'medium';
			$product_image_width  = isset($view_options['catsp_catspup_image_crop_size']['top']) ? $view_options['catsp_catspup_image_crop_size']['top'] : '';
			$product_image_height = isset($view_options['catsp_catspup_image_crop_size']['right']) ? $view_options['catsp_catspup_image_crop_size']['right'] : '';
			$product_image_crop   = isset($view_options['catsp_catspup_image_crop_size']['style']) ? $view_options['catsp_catspup_image_crop_size']['style'] : '';
			$thumb_full_src       = wp_get_attachment_image_src($thumb_id, 'full');
			$thumb_full_src       = is_array($thumb_full_src) ? $thumb_full_src : array('', '', '');
			$image_src            = wp_get_attachment_image_src($thumb_id, $image_sizes);
			$image_src            = is_array($image_src) ? $image_src : array('', '', '');
			if (('custom' === $image_sizes) && (!empty($product_image_width) && $thumb_full_src[1] >= $product_image_width) && (!empty($product_image_height) && $thumb_full_src[2] >= $product_image_height)) {
				$hard_crop = 'Hard-crop' === $product_image_crop ? true : false;

				$image        = ta_catsp_resize($thumb_full_src[0], $product_image_width, $product_image_height, $hard_crop);
				$image_width  = $product_image_width;
				$image_height = $product_image_height;
			} else {
				$image        = !empty($image_src[0]) ? $image_src[0] : '';
				$image_width  = !empty($image_src[1]) ? $image_src[1] : 600;
				$image_height = !empty($image_src[2]) ? $image_src[2] : 450;
			}
		}
		return array(
			'src'    => $image,
			'width'  => $image_width,
			'height' => $image_height,
		);
	}


	/**
	 * Maximum pages.
	 *
	 * @param int $total_post Number of total posts.
	 * @param int $product_per_page Posts per page.
	 *
	 * @return void
	 */
	public static function catsp_max_pages($total_post, $product_per_page)
	{
		if (!$total_post) {
			return;
		}
		$max_num_pages = ceil($total_post / $product_per_page);
		return (int) $max_num_pages;
	}


	/**
	 * Pagination last page post
	 *
	 * @param [type] $limit total post limit.
	 * @param [type] $product_per_page post per page.
	 * @param [type] $total_page last post page.
	 *
	 * @return int.
	 */
	public static function catsp_last_page_post($limit, $product_per_page, $total_page)
	{
		$limit              = (empty($limit) || '-1' === $limit) ? 10000000 : $limit;
		$offset             = $product_per_page * ($total_page - 1);
		$catsp_last_page_post = $limit - $offset;
		return $catsp_last_page_post;
	}

	/**
	 * Get view option from view ID
	 *
	 * @param string $catsp_gl_id ID of custom field.
	 *
	 * @return array
	 */
	public static function view_options($catsp_gl_id)
	{
		if (!$catsp_gl_id) {
			return;
		}
		$view_options = get_post_meta($catsp_gl_id, 'ta_catsp_view_options', true);
		return $view_options;
	}

	/**
	 * Get value of a setting from global settings array
	 *
	 * @param string     $field        The full name of setting to get value.
	 * @param array      $array_to_get Array to get values of wanted setting.
	 * @param mixed|null $assign       The value to assign if setting is not found.
	 */
	public static function catsp_metabox_value($field, $array_to_get = null, $assign = null)
	{
		global $catsp_gl_id;
		if (empty($array_to_get)) {
			$array_to_get = self::view_options($catsp_gl_id);
		}
		return isset($array_to_get[$field]) ? $array_to_get[$field] : $assign;
	}

	/**
	 * Convert the slug text to a name.
	 *
	 * @param string $slug The slug string.
	 * @return string
	 */
	public static function slug_string_to_name($slug)
	{
		$slug = preg_replace('/[_\-]+/', ' ', $slug);

		return ucwords($slug);
	}

	/**
	 * Custom Template locator .
	 *
	 * @param  mixed $template_name template name .
	 * @param  mixed $template_path template path .
	 * @param  mixed $default_path default path .
	 * @return string
	 */
	public static function catsp_locate_template($template_name, $template_path = '', $default_path = '')
	{
		if (!$template_path) {
			$template_path = 'catshowcase-pro/templates';
		}

		if (!$default_path) {
			$default_path = CATSP_PATH . 'public/templates/';
		}
		$template = locate_template(trailingslashit($template_path) . $template_name);
		// Get default template.
		if (!$template) {
			$template = $default_path . $template_name;
		}
		// Return what we found.
		return $template;
	}
}
